FEATURE = "Feature"
BACKGROUND = "Background"
SCENARIO = "Scenario"
GIVEN = "Given"
WHEN = "When"
THEN = "Then"
AND = "And"
BUT = "But"

FEATURE_CS = "Požadavek"
BACKGROUND_CS = "Kontext"
SCENARIO_CS = "Scénář"
GIVEN_CS = "Pokud"
WHEN_CS = "Když"
THEN_CS = "Pak"
AND_CS = "A"
BUT_CS = "Ale"